#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Contact :   liuyuqi.gov@msn.cn
@Time    :   2023/09/14 07:14:07
@License :   Copyright © 2017-2022 liuyuqi. All Rights Reserved.
@Desc    :   
https://www.qinimg.com/
'''
import requests
import sys
import os
import re
import json,time
from bs4 import BeautifulSoup
import sqlite3
from concurrent.futures import ThreadPoolExecutor

class Qinimg(object):
    '''  '''
    _host = "https://www.qinimg.com"
    # _host="http://localhost:88"
    
    _header = {
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36(KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36",
               "Referer": "https://www.qinimg.com/",
               "authority": "www.qinimg.com",
                "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
                "accept-language": "en-US,en;q=0.9",
                "cache-control": "max-age=0",
                "dnt": "1",
                "sec-ch-ua": '"Chromium";v="116", "Not)A;Brand";v="24", "Microsoft Edge";v="116"',
                "sec-ch-ua-mobile": "?0",
                "sec-ch-ua-platform": '"Windows"',
                "sec-fetch-dest": "document",
                "sec-fetch-mode": "navigate",
                "sec-fetch-site": "same-origin",
                "sec-fetch-user": "?1",
                "upgrade-insecure-requests": "1"
               }
    max_page = 10

    def __init__(self):
        self.sess = requests.Session()
        # set proxy
        # self.sess.proxies = {"http": "http://127.0.0.1:2181", "https": "http://127.0.0.1:2181"}
        self.conn = sqlite3.connect('xiaohua.db')
        self.cursor = self.conn.cursor()
        self.init_database()
        
    def init_database(self):
        ''' init sqlite database '''
        # whether the ''qinimg_girl table exists
        self.cursor.execute("SELECT name FROM sqlite_master WHERE type='table' AND name='qinimg_girl'")
        exists = self.cursor.fetchone()
        if not exists:
        # if not exist, create table
            sql1='''
    CREATE TABLE "qinimg_girl" (
    "id"  INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
    "name"  TEXT,
    "uid"  INTEGER,
    "createtime"  TEXT,
    "isdelete"  INTEGER NOT NULL DEFAULT 0,
    "url"  TEXT
    );
    '''
            self.cursor.execute(sql1)

        self.cursor.execute("SELECT name FROM sqlite_master WHERE type='table' AND name='qinimg_girl_detail'")
        exists = self.cursor.fetchone()
        if not exists:
            sql2='''
    CREATE TABLE "qinimg_girl_detail" (
    "id"  INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
    "url"  TEXT,
    "is_download"  INTEGER NOT NULL DEFAULT 0,
    "createtime"  TEXT
    );
            '''
            self.cursor.execute(sql2)
        
        # self.cursor.close()
        # self.conn.close()

    def get_list(self, next_page=None):
        ''' get all pic list '''
        print("init page: " + next_page)
        if next_page is None:
            next_page = self._host
        else:
            next_page = next_page

        res=self.sess.get(next_page, headers=self._header)
        res.encoding = 'utf-8'
        soup = BeautifulSoup(res.text, 'html.parser')
        imgs = soup.select('div.list_box ul')
        # save to sqlite, qinimg_girl
        for img in imgs:
            try:
                url=img.select('a')[0]['href'].strip()
                title=img.select('a')[0]['title'].strip()
                createtime=img.select('span')[1].text.strip()
                self.save_girl(url,title,createtime)
                self.get_pic_detail(url,title,createtime)
            except Exception as e:
                print(e)
            finally:
                pass
        
        # find the next button
        pagenumbers=soup.select('div.pages a')
        index=0
        for pagenumber in pagenumbers:
            if pagenumber.text == 'Next' and index < self.max_page:
                next = pagenumber
                next_page = self._host + next['href']
                self.get_list(next_page=next_page)
                break
            index=index + 1
        self.set_config("init_page", next_page)

    def save_girl(self, url:str, title:str, createtime:str):
        ''' save girl list to sqlite '''
        # if not exist in sqlite, add to sqlite
        # if exist, skip
        uid=re.findall(r'(\d+)', url)[0]
        url=self._host+url
        sql1='''
Select * from qinimg_girl where uid = '{}' and isdelete = 0
'''
        # print(sql1.format(uid))
        self.cursor.execute(sql1.format(str(uid)))
        exists = self.cursor.fetchone()
        if not exists:
            sql2='''
    INSERT INTO qinimg_girl ("name", "uid", "createtime", "url") 
    VALUES ("{}", {}, "{}",  "{}");
    '''
            # print(sql2.format(title, uid, createtime, url))
            self.cursor.execute(sql2.format(title, uid, createtime, url))
            self.conn.commit()
        else:
            print(str(uid)+" is exist.")

    def get_pic_detail(self, url, name, createtime):
        ''' get pic detail '''
        uid = re.findall(r'(\d+)', url)[0]
        pic_index = 0
        url = self._host + url
        try:
            res=self.sess.get(url, headers=self._header)
            res.encoding = 'utf-8'
            soup = BeautifulSoup(res.text, 'html.parser')
            # find all img
            imgs=soup.select('div#image p a')
            # save to sqlite, qinimg_girl_detail
            for img in imgs:
                # if not exist in sqlite, add to sqlite
                # if exist, check whether the pic is downloaded
                # self.download_all_pic(img['href'])
                self.save_girl_detail(uid, name, pic_index, createtime, img['href'])
                pic_index = pic_index + 1
        except Exception as e:
            print(e)
        finally:
            pass


    def save_girl_detail(self, uid,name,pic_index,createtime,url):
        ''' save girl detail to sqlite '''
        sql='''
insert into qinimg_girl_detail (uid,name,pic_index,createtime,url) values ('{}', '{}', '{}', '{}', '{}')
        '''
        self.cursor.execute(sql.format(uid,name,str(pic_index),createtime,url))
        self.conn.commit()
        # print(sql.format(uid,name,str(pic_index),createtime,url))

    def download_all_pic(self):
        ''' download pic '''
        sql = '''
        select * from qinimg_girl_detail where is_download = 0
        '''
        # get all pic detail
        self.cursor.execute(sql)
        details = self.cursor.fetchall()
        index=1
        for detail in details:
            pool = ThreadPoolExecutor(max_workers=5)
            future1 = pool.submit(self._download_pic, detail[4], detail[2], detail[3])
            # print(detail)
            # if index>2:
            #     return
            # index = index + 1
            time.sleep(1)

    def _download_pic(self, url, name,index):
        ''' download pic '''
        if not os.path.exists("data"):
            os.mkdir("data")
        if not os.path.exists("data/{}".format(name)):
            os.mkdir("data/{}".format(name))
        with open("data/{}/{}.jpg".format(name, index), "wb") as f:
            header ={
                "authority": "qinimg.woshitu.cfd",
                "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
                "accept-language": "en-US,en;q=0.9",
                "cache-control": "max-age=0",
                "dnt": "1",
                "if-modified-since": "Wed, 13 Sep 2023 00:24:47 GMT",
                "if-none-match": '"6501014f-6e9f"',
                "referer": "https://www.qinimg.com/",
                "sec-ch-ua": '"Chromium";v="116", "Not)A;Brand";v="24", "Microsoft Edge";v="116"',
                "sec-ch-ua-mobile": "?0",
                "sec-ch-ua-platform": '"Windows"',
                "sec-fetch-dest": "document",
                "sec-fetch-mode": "navigate",
                "sec-fetch-site": "cross-site",
                "sec-fetch-user": "?1",
                "upgrade-insecure-requests": "1",
                "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36(KHTML, like Gecko) Chrome/116.0.0.0 Safari/537.36 Edg/116.0.1938.76"
                }
            res = self.sess.get(url, headers=header)
            f.write(res.content)
            print("download pic: " + url)

        sql='''
        UPDATE qinimg_girl_detail set is_download = 1 where url = '{}'
        '''
        self.cursor.execute(sql.format(url))
        self.conn.commit()

    def run(self):
        ''' run '''
        # get all pic
        init_page = self.get_config("init_page")
        self.get_list(next_page = init_page)
        # download pic
        # self.download_all_pic()
        self.__release__()

    def set_config(self, key, value):
        ''' save config '''
        if self.get_config(key) is None:
            sql='''
    insert into qinimg_config (key, value) values ('{}', '{}')
    '''
        else:
            sql='''
    update qinimg_config set value = '{}' where key = '{}'
    '''
        try:
            self.cursor.execute(sql.format(key, value))
            self.conn.commit()
        except Exception as e:
            print(e)
        finally:
            pass

    
    def get_config(self, key):
        ''' get config '''
        sql='''
select * from qinimg_config where key = '{}'
'''     
        try:
            self.cursor.execute(sql.format(key))
            config = self.cursor.fetchone()
            return config[2]
        except Exception as e:
            return None
        finally:
            pass

    def __release__(self):
        ''' release '''
        self.cursor.close()
        self.conn.close()


class Girl:
    ''' girl model '''
    def __init__(self, name, uid, createtime, url, isdelete=0):
        self.name = name
        self.uid = uid
        self.createtime = createtime
        self.url = url
        self.isdelete = isdelete

class GirlDetail:
    ''' girl detail model '''  
    def __init__(self, uid, name, pic_index, url, is_download, createtime):
        self.uid = uid
        self.name = name
        self.pic_index = pic_index
        self.url = url
        self.is_download = is_download
        self.createtime = createtime
    
class Config:
    ''' config model '''
    def __init__(self, key, value, isdelete=0, desc=""):
        self.key = key
        self.value = value
        self.isdelete=isdelete
        self.desc=desc

if __name__ == "__main__":
    qinimg = Qinimg()
    qinimg.run()
