#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Contact :   liuyuqi.gov@msn.cn
@Time    :   2023/09/21 14:12:51
@License :   Copyright © 2017-2022 liuyuqi. All Rights Reserved.
@Desc    :   校花网
'''
from crawl_xiaohua.extractor.base_extractor import BaseExtractor
from contextlib import closing
import os
import random
import time
from crawl_xiaohua.libs.json_conf import JsonConf
import requests
from crawl_xiaohua import api
import bs4
import pandas as pd

class Xiaohua(BaseExtractor):
    
    _header={
        "Authority": "img.xiaohua.com",
        "Accept": "image/webp,image/apng,image/svg+xml,image/*,*/*;q=0.8",
        "Accept-Language": "en-US,en;q=0.9,zh-CN;q=0.8,zh;q=0.7",
        "Dnt": "1",
        "Referer": "http://www.xiaohua.com/",
        "Sec-Ch-Ua-Mobile": "?0",
        "Sec-Ch-Ua-Platform": "Windows",
        "Sec-Fetch-Dest": "image",
        "Sec-Fetch-Mode": "no-cors",
        "Sec-Fetch-Site": "cross-site",
    }

    def __init__(self):
        self.s = requests.Session()
        self.s.headers.update(self._header)
        self.jsonConf = JsonConf()
        self.conf = self.jsonConf.load()
        self.indexPage = self.conf.get('indexPage')
        self.indexDuanziPage = self.conf.get('indexDuanziPage')
        # self.s.cookies.update(JsonConf().get_cookies())

    def crawl(self):
        for i in range(10000):
            self.getPicList()
            time.sleep(random.randint(1, 5))

    def crawlDuanzi(self):
        for i in range(10000):
            self.getDuanziList()
            self.indexDuanziPage = str(i)
            self.jsonConf.set({"indexDuanziPage": self.indexDuanziPage})
            time.sleep(random.randint(1, 5))

    def getDuanziList(self):
        res = self.s.get(api.startDuanziUrl + "?page=" + self.indexDuanziPage)
        resHtml = bs4.BeautifulSoup(res.text, 'html.parser')
        divContentLeft = resHtml.find_all('div', {
            'class': 'one-cont'})
        divContentLefts = [div.find('p').find(
            'a').text for div in divContentLeft]
        self.saveDuanZiList(divContentLefts)

    def saveDuanZiList(self, duanziList):
        pd.DataFrame(duanziList).to_csv(r"data/duanzhi.csv",
                                        mode='a', encoding='utf-8', header=False)

    def getPicList(self):
        res = self.s.get(api.startUrl + self.indexPage)
        resHtml = bs4.BeautifulSoup(res.text, 'html.parser')
        divPic = resHtml.find('div', {'id': 'divPic'})
        divContentLeft = resHtml.find('div', {'class': 'content-left'})
        imgDiv = divPic.find_all('img')
        imgs = [img.get('data-src') for img in imgDiv]
        titleDesc = divContentLeft.find('p').text
        titleDesc = titleDesc if len(titleDesc) <= 20 else titleDesc[:10]
        # 下载图片
        for i in range(len(imgs)):
            self.downloadPic(imgs[i], titleDesc + "_" + self.indexPage + "_" +
                             str(i) + "." + imgs[0].split('.')[-1])

        self.jsonConf.set({"indexPage": self.indexPage})
        # 下一页
        nextPan = str.split(resHtml.find(
            'a', {'id': 'hylPrev'})["href"], r"/")[2]
        self.indexPage = nextPan
        return nextPan

    def downloadPic(self, imgUrl, title):
        with closing(self.s.get(url=imgUrl, stream=True)) as response:
            chunk_size = 1024
            file_D = './data/' + title
            if (os.path.exists(file_D)):
                print('跳过' + title)
            else:
                with open(file_D, "wb") as file:
                    for data in response.iter_content(chunk_size=chunk_size):
                        file.write(data)

    def getCookie(self):
        pass
